#!/usr/bin/perl -w

# lpcboot - an LPC17xx MMC/SD boot loader inspired by HolgerBootloader2
# Based on a heavily modified version of ChaN's FatFs library
#
# Copyright (C) 2013  Ingo Korb <ingo@akana.de>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
#
#   lpc_checksum.pl: Vector checksum generator for LPC17xx binaries
#

use strict;

if (scalar(@ARGV) != 1) {
    print "Usage: $0 binfile\n";
    exit 1;
}

open FD,"+<",$ARGV[0] or die "Can't open $ARGV[0]: $!";

my $buffer;
if (sysread(FD, $buffer, 4*7) != 4*7) {
    die "Short read from file: $!";
}

my @vecs = unpack("V*", $buffer);
my $checksum = 0;

while (scalar(@vecs) > 0) {
    $checksum += pop(@vecs);
}

$checksum = (-$checksum) & 0xffffffff;

if (syswrite(FD, pack("V", $checksum), 4) != 4) {
    die "Short write to file: $!";
}

close FD;
